@@ -1,5 +1,5 @@
 /*
-	$Id: acconfig.h,v 1.4 1999/11/17 13:48:08 crosser Exp $
+	$Id: acconfig.h,v 1.7 2000/02/04 22:31:56 crosser Exp $
 */
 
 /*
@@ -17,6 +17,18 @@
 
 /*
 	$Log: acconfig.h,v $
+	Revision 1.7  2000/02/04 22:31:56  crosser
+	Clean up things dealing with folder tree recursion.
+	Fixed log entry in acconfig.h that conflicted with "configure"
+	
+	Revision 1.6  2000/02/01 22:04:15  crosser
+	xchdir() function iterates thru path elements
+	define a variable for extra verbosity
+	
+	Revision 1.5  1999/12/11 14:10:15  crosser
+	Support sgtty terminal control
+	Proper "fake speed" handling (needed two values)
+	
 	Revision 1.4  1999/11/17 13:48:08  crosser
 	check if need to include getopt.h
 	
@@ -53,3 +65,14 @@
 
 /* headers contain definition of optarg */
 #undef OPTARG_DEFINED
+
+/* terminal control type */
+#undef USE_TERMIOS
+#undef USE_SGTTY
+#undef USE_TERMIO
+
+/* is struct utimbuf defined in sys/utime.h */
+#undef HAVE_UTIMBUF
+
+/* devine for development versions - extra verbosity */
+#undef DEVVERSION
