@@ -60,3 +60,6 @@
 /* Define if compiler has function prototypes */
 #define PROTOTYPES 1
 
+/* is struct utimbuf defined in sys/utime.h */
+#define HAVE_UTIMBUF 1
+
