@@ -1,5 +1,5 @@
 #ifndef LINT
-static char *rcsid="$Id: eph_cmd.c,v 1.15 1999/08/01 21:36:54 crosser Exp $";
+static char *rcsid="$Id: eph_cmd.c,v 1.16 2000/02/13 11:15:01 crosser Exp $";
 #endif
 
 /*
@@ -17,6 +17,9 @@ static char *rcsid="$Id: eph_cmd.c,v 1.15 1999/08/01 21:36:54 crosser Exp $";
 
 /*
 	$Log: eph_cmd.c,v $
+	Revision 1.16  2000/02/13 11:15:01  crosser
+	Kludge null setint for Nikon
+	
 	Revision 1.15  1999/08/01 21:36:54  crosser
 	Modify source to suit ansi2knr
 	(I hate the style that ansi2knr requires but you don't expect me
@@ -138,6 +141,27 @@ writeagain:
 	return rc;
 }
 
+int
+eph_setnullint(eph_iob *iob,int reg)
+{
+	unsigned char buf[2];
+	int rc;
+	int count=0;
+
+	buf[0]=CMD_SETINT;
+	buf[1]=reg;
+
+writeagain:
+	if ((rc=eph_writecmd(iob,buf,2))) return rc;
+	rc=eph_waitack(iob,(reg == REG_FRAME)?BIGACKTIMEOUT:ACKTIMEOUT);
+	if (MAYRETRY(rc) && (count++ < RETRIES)) goto writeagain;
+	if (count >= RETRIES)
+		eph_error(iob,ERR_EXCESSIVE_RETRY,
+				"excessive retries on setnullint");
+
+	return rc;
+}
+
 int
 eph_getint(eph_iob *iob,int reg,long *val)
 {
