@@ -1,5 +1,5 @@
 #ifndef LINT
-static char *rcsid="$Id: eph_io.c,v 2.16 1999/08/01 21:36:54 crosser Exp $";
+static char *rcsid="$Id: eph_io.c,v 2.17 2000/02/17 21:29:54 crosser Exp $";
 #endif
 
 /*
@@ -17,6 +17,10 @@ static char *rcsid="$Id: eph_io.c,v 2.16 1999/08/01 21:36:54 crosser Exp $";
 
 /*
 	$Log: eph_io.c,v $
+	Revision 2.17  2000/02/17 21:29:54  crosser
+	final cleanup for 3.04, change version
+	change debugging levels
+	
 	Revision 2.16  1999/08/01 21:36:54  crosser
 	Modify source to suit ansi2knr
 	(I hate the style that ansi2knr requires but you don't expect me
@@ -222,7 +226,7 @@ eph_writepkt(eph_iob *iob,int typ,int seq,char *data,size_t length)
 	}
 	buf[i++]=crc&0xff;
 	buf[i++]=crc>>8;
-	if (iob->debug) {
+	if (iob->debug > 1) {
 		printf("> (%d)",i);
 		for (j=0;j<i;j++) {
 			printf(" %02x",buf[j]);
@@ -277,7 +281,7 @@ eph_writeinit(eph_iob *iob)
 {
 	unsigned char init=0;
 
-	if (iob->debug) printf("> INIT 00\n");
+	if (iob->debug > 1) printf("> INIT 00\n");
 	shortsleep(WRTDELAY);
 	if (WRITE(iob->fd,&init,1)) {
 		eph_error(iob,ERRNO,"init write error %s",strerror(ERRNO));
@@ -289,7 +293,7 @@ eph_writeack(eph_iob *iob)
 {
 	unsigned char ack=ACK;
 
-	if (iob->debug) printf("> ACK 06\n");
+	if (iob->debug > 1) printf("> ACK 06\n");
 	shortsleep(WRTDELAY);
 	if (WRITE(iob->fd,&ack,1)) {
 		eph_error(iob,ERRNO,"ack write errot %s",strerror(ERRNO));
@@ -301,7 +305,7 @@ eph_writenak(eph_iob *iob)
 {
 	unsigned char nak=NAK;
 
-	if (iob->debug) printf("> NAK 11\n");
+	if (iob->debug > 1) printf("> NAK 11\n");
 	shortsleep(WRTDELAY);
 	if (WRITE(iob->fd,&nak,1)) {
 		eph_error(iob,ERRNO,"nak write error %s",strerror(ERRNO));
@@ -318,7 +322,7 @@ eph_readpkt(eph_iob *iob,eph_pkthdr *pkthdr,char *buffer,
 	int i,rc;
 
 	i=eph_readt(iob,buf,1,timeout_usec,&rc);
-	if (iob->debug)
+	if (iob->debug > 1)
 		printf ("pktstart: i=%d rc=%d char=0x%02x\n",i,rc,*buf);
 	if (i < 0) {
 		eph_error(iob,ERRNO,"pkt start read error %s",strerror(ERRNO));
@@ -356,7 +360,7 @@ eph_readpkt(eph_iob *iob,eph_pkthdr *pkthdr,char *buffer,
 			return -1;
 		}
 	}
-	if (iob->debug) printf("header: %02x %02x %02x %02x\n",
+	if (iob->debug > 1) printf("header: %02x %02x %02x %02x\n",
 				buf[0],buf[1],buf[2],buf[3]);
 	pkthdr->seq=buf[1];
 	length=(buf[3]<<8)|buf[2];
@@ -394,7 +398,7 @@ eph_readpkt(eph_iob *iob,eph_pkthdr *pkthdr,char *buffer,
 	while ((i=eph_readt(iob,buf+got,2-got,DATATIMEOUT,&rc)) > 0) {
 		got+=i;
 	}
-	if (iob->debug)
+	if (iob->debug > 1)
 		printf ("crc: %02x %02x i=%d rc=%d\n",buf[0],buf[1],i,rc);
 	if (got != 2) {
 		if (i < 0) {
@@ -418,16 +422,16 @@ eph_readpkt(eph_iob *iob,eph_pkthdr *pkthdr,char *buffer,
 			"crc received=0x%04x counted=0x%04x",crc2,crc1);
 		return -1;
 	}
-	if (iob->debug) {
+	if (iob->debug > 1) {
 		int j;
 
 		printf("< %d,%d (%d)",pkthdr->typ,pkthdr->seq,length);
-		if (iob->debug > 1) for (j=0;j<length;j++) {
+		if (iob->debug > 2) for (j=0;j<length;j++) {
 			printf(" %02x",(unsigned char)buffer[j]);
 		} else printf(" ...");
 		printf("\n");
 		printf("< %d,%d (%d)",pkthdr->typ,pkthdr->seq,length);
-		if (iob->debug > 1) for (j=0;j<length;j++) {
+		if (iob->debug > 2) for (j=0;j<length;j++) {
 			printf(" %c ",(buffer[j] >= ' ' && buffer[j] < 127)
 							? buffer[j] : '.');
 		} else printf(" ...");
@@ -444,7 +448,7 @@ eph_flushinput(eph_iob *iob)
 	int i,rc;
 
 	i=eph_readt(iob,&buf,1,0L,&rc);
-	if (iob->debug)
+	if (iob->debug > 1)
 		printf ("< %02x amount=%d rc=%d\n",buf,i,rc);
 	if (i < 0) {
 		eph_error(iob,ERRNO,"flushinput read error %s",strerror(ERRNO));
@@ -466,7 +470,7 @@ eph_waitchar(eph_iob *iob,unsigned long timeout_usec)
 	int i,rc;
 
 	i=eph_readt(iob,&buf,1,timeout_usec,&rc);
-	if (iob->debug)
+	if (iob->debug > 1)
 		printf ("< %02x amount=%d rc=%d\n",buf,i,rc);
 	if (i < 0) {
 		eph_error(iob,ERRNO,"waitchar read error %s",strerror(ERRNO));
