@@ -1,5 +1,5 @@
 /*
-	$Id: eph_io.h,v 2.7 1998/10/18 13:18:27 crosser Exp $
+	$Id: eph_io.h,v 2.10 2000/02/13 11:15:01 crosser Exp $
 */
 
 /*
@@ -17,6 +17,16 @@
 
 /*
 	$Log: eph_io.h,v $
+	Revision 2.10  2000/02/13 11:15:01  crosser
+	Kludge null setint for Nikon
+	
+	Revision 2.9  1999/12/11 14:10:15  crosser
+	Support sgtty terminal control
+	Proper "fake speed" handling (needed two values)
+	
+	Revision 2.8  1999/12/01 21:41:23  crosser
+	add "pseudo" speed
+	
 	Revision 2.7  1998/10/18 13:18:27  crosser
 	Put RCS logs and I.D. into the source
 
@@ -54,7 +64,16 @@
 typedef long off_t;
 #endif
 #if defined(UNIX)
-#include <termios.h>
+#if defined(USE_TERMIOS)
+# include <termios.h>
+#elif defined(USE_SGTTY)
+# include <sgtty.h>
+#elif defined(USE_TERMIO)
+#include <termio.h>
+ # error "termio unsupported, sorry"
+#else
+ # error "no termios, sgtty or termio defined, no way to control the tty"
+#endif
 #elif defined(MSWINDOWS)
 #include <windows.h>
 #endif
@@ -74,7 +93,13 @@ typedef struct _eph_iob {
 	int debug;
 #if defined(UNIX)
 	int fd;
+#if defined(USE_TERMIOS)
 	struct termios savetios;
+#elif defined(USE_SGTTY)
+	struct sgttyb savesgtty;
+#elif defined(USE_TERMIO)
+	struct termio savetio;
+#endif
 #elif defined(MSWINDOWS)
 	HANDLE fd;
 	DCB savedcb;
@@ -90,11 +115,13 @@ eph_iob *eph_new(void (*errorcb)(int errcode,char *errstr),
 		void (*runcb)(off_t count),
 		int (*storecb)(char *data,size_t size),
 		int debug);
-int eph_open(eph_iob *iob,char *device_name,long speed);
+int eph_open(eph_iob *iob,char *device_name,long speed,
+		long defttspeed,long ttspeed);
 int eph_close(eph_iob *iob,int newmodel);
 void eph_free(eph_iob *iob);
 
 int eph_setint(eph_iob *iob,int reg,long val);
+int eph_setnullint(eph_iob *iob,int reg);
 int eph_getint(eph_iob *iob,int reg,long *val);
 int eph_action(eph_iob *iob,int reg,char *val,size_t length);
 int eph_setvar(eph_iob *iob,int reg,char *val,off_t length);
