@@ -1,5 +1,5 @@
 #ifndef LINT
-static char *rcsid="$Id: photopc.c,v 2.56 1999/11/17 13:48:08 crosser Exp $";
+static char *rcsid="$Id: photopc.c,v 2.74 2000/02/17 21:29:55 crosser Exp $";
 #endif
 
 /*
@@ -17,6 +17,65 @@ static char *rcsid="$Id: photopc.c,v 2.56 1999/11/17 13:48:08 crosser Exp $";
 
 /*
 	$Log: photopc.c,v $
+	Revision 2.74  2000/02/17 21:29:55  crosser
+	final cleanup for 3.04, change version
+	change debugging levels
+	
+	Revision 2.73  2000/02/17 07:07:41  crosser
+	cleanup for upcoming release
+	
+	Revision 2.72  2000/02/16 22:55:57  crosser
+	backout some changes
+	
+	Revision 2.70  2000/02/16 22:04:03  crosser
+	playing with order of things for Nikon...
+	Do not print header if directory is empty.
+	
+	Revision 2.69  2000/02/13 11:15:01  crosser
+	Kludge null setint for Nikon
+	
+	Revision 2.68  2000/02/08 06:09:54  crosser
+	properly pass back return code through invocations of descend()
+	
+	Revision 2.67  2000/02/05 12:37:42  crosser
+	Oops, resore the code that I have accidently deleted...
+	
+	Revision 2.66  2000/02/04 22:31:56  crosser
+	Clean up things dealing with folder tree recursion.
+	Fixed log entry in acconfig.h that conflicted with "configure"
+	
+	Revision 2.65  2000/02/01 22:04:15  crosser
+	xchdir() function iterates thru path elements
+	define DEVVERSION for extra verbosity
+	
+	Revision 2.64  2000/01/31 21:59:31  crosser
+	Oops!  trailing slash...
+	
+	Revision 2.63  2000/01/25 22:16:26  crosser
+	more work trying to chdir properly.  I wonder if chdir to multi-component
+	path works at all.  Now when path is built properly we'll be able to
+	check it.  If chdir-ing to something like "\DCIM\1000OLYM" is impossible
+	we'll have to descend recursively from root every time!  Oh boy :-(
+	
+	Revision 2.62  2000/01/24 20:25:23  crosser
+	do better job in walking thru folder tree; credit to Sergey I. Panov
+	
+	Revision 2.61  2000/01/23 15:51:02  crosser
+	work in progress on folder tree support
+	
+	Revision 2.60  2000/01/21 15:03:25  crosser
+	Experimenting with support for models with folder structure
+	
+	Revision 2.59  1999/12/26 13:59:45  crosser
+	sort frame list for erase command; new version 3.03
+	
+	Revision 2.58  1999/12/11 14:10:15  crosser
+	Support sgtty terminal control
+	Proper "fake speed" handling (needed two values)
+	
+	Revision 2.57  1999/12/01 21:41:24  crosser
+	add "pseudo" speed
+	
 	Revision 2.56  1999/11/17 13:48:08  crosser
 	check if need to include getopt.h
 	
@@ -269,6 +328,11 @@ static char *rcsid="$Id: photopc.c,v 2.56 1999/11/17 13:48:08 crosser Exp $";
 #include "ctimez.h"
 #include "filetime.h"
 
+#ifdef MAXPATH
+#undef MAXPATH
+#endif
+#define MAXPATH 256
+
 #ifdef DOS
 #define MS_PROGRAM_NAMES
 #pragma warn -par
@@ -309,6 +373,7 @@ static char *nameformat=NULL;
 static int switchoff=0;
 static FILE *fp = NULL;
 static char *fname = NULL;
+static int have_folders=0;
 
 void
 file_abort_cleanup(void)
@@ -325,13 +390,126 @@ file_abort_cleanup(void)
 	}
 }
 
+static int
+xchdir(eph_iob *iob,char *where)
+{
+	char path[MAXPATH],*p;
+
+	if (!have_folders) {
+		fprintf(stderr,"no folders on this model\n");
+		return -1;
+	}
+	strncpy(path,where,sizeof(path)-1);
+	path[sizeof(path)-1]='\0';
+	for (p=path;*p;p++) if (*p == '/') *p='\\';
+	if (path[0] == '\\') {
+		if (eph_setvar(iob,84,"\\",1)) {
+			fprintf(stderr,"could not chdir to \"\\\"\n");
+			return -1;
+		}
+	}
+	for (p=strtok(path,"\\");p;p=strtok(NULL,"\\")) {
+		if (eph_setvar(iob,84,p,strlen(p))) {
+			fprintf(stderr,"could not chdir to \"%s\"\n",p);
+			return -1;
+		}
+	}
+	return 0;
+}
+
+static int
+descend(eph_iob *iob,int depth,char *root,int(*each)(eph_iob *iob,char *path))
+{
+	long nfolders=0L,i;
+	off_t f_size;
+	char *folder,*p;
+	char path[MAXPATH];
+	int rc;
+
+	if (!have_folders) return (*each)(iob,root);
+
+	if (depth > 10) {
+		fprintf(stderr,"cannot be that deep (%d), error!\n",depth);
+		return -1;
+	}
+	if (xchdir(iob,root)) {
+		fprintf(stderr,"chdir to \"%s\" failed\n",root);
+		return -1;
+	}
+
+	if ((rc=(*each)(iob,root)) != 0) {
+		if (debug) printf("(*each)() returned %d\n",rc);
+		return rc;
+	}
+
+	eph_getint(iob,83,&nfolders);
+	if (debug) printf("descend: %s has %ld folders\n",
+				root,nfolders);
+	if (nfolders == 0L) return 0;
+
+	folder=(char*)malloc(2048);
+	f_size=2048;
+	path[sizeof(path)-1]='\0';
+	for (i=1;i<=nfolders;i++) {
+		eph_setint(iob,83,i);
+		eph_getvar(iob,84,&folder,&f_size);
+		if (debug) printf("depth=%4d i=%4ld:\n",depth,i);
+		strncpy(path,root,sizeof(path)-2);
+		if (path[strlen(path)-1] != '\\')
+			strncat(path,"\\",sizeof(path)-2);
+		strncat(path,folder,sizeof(path)-2);
+		for (p=path+strlen(path)-1;(*p == ' ') && (p > path);p--)
+			*p='\0';
+		if ((rc=descend(iob,depth+1,path,each))) {
+			free(folder);
+			return rc;
+		}
+		if (xchdir(iob,root)) {
+			fprintf(stderr,"restore dir to \"%s\" failed\n",root);
+			return -1;
+		}
+	}
+	free(folder);
+	if (debug) printf("descend: full tree walked through\n");
+	return 0;
+}
+
+static int
+nonempty(eph_iob *iob,char *path)
+{
+	unsigned long result;
+
+	if (eph_getint(iob,10,&result)) {
+		return -1;
+	}
+	if (result) {
+		if (!quiet) printf("Starting in folder \"%s\"\n",path);
+		return 1;
+	} else return 0;
+}
+
 int
-probe(eph_iob *iob)
+init(eph_iob *iob)
 {
 	long ret;
+	int rc;
 
 	if (eph_getint(iob,1,&ret)) return -1;
-	else return 0;
+#if 0
+	(void)eph_setint(iob,77,1L);
+	(void)eph_setint(iob,82,60L);
+#endif
+	if ((rc=eph_setnullint(iob,83))) {
+		if (debug) printf("setnullint(83) returned %d\n",rc);
+	}
+	if (eph_setvar(iob,84,"\\",1)) {
+		if (debug) printf("No folders on this model\n");
+	} else {
+		have_folders=1;
+	}
+	if (have_folders)
+		if (descend(iob,0,"\\",nonempty) < 0) return -1;
+	return 0;
 }
 
 int
@@ -814,7 +992,7 @@ count(eph_iob *iob,int argc,char *argv[])
 {
 	unsigned long result;
 
-	if (eph_getint(iob,0x0a,&result)) {
+	if (eph_getint(iob,10,&result)) {
 		printf("-1\n");
 		return -1;
 	}
@@ -979,7 +1157,7 @@ allregs(eph_iob *iob,int argc,char *argv[])
 }
 
 int
-list(eph_iob *iob,int argc,char *argv[])
+listfiles(eph_iob *iob,char *path)
 {
 	char *buffer;
 	off_t bufsize;
@@ -989,8 +1167,13 @@ list(eph_iob *iob,int argc,char *argv[])
 	if (eph_getint(iob,10,&max)) {
 		return -1;
 	}
-	printf("total %lu\n",max);
-	printf("No.    Size R P Date and Time\n");
+	if (have_folders) printf("%s: ",path);
+	if (max)
+		printf("total %lu\n"
+		"No.    Size R P Date and Time                Filename\n",
+			max);
+	else
+		printf("\n");
 
 	for (j=1;j<=max;j++) {
 		printf("%3ld ",j);
@@ -1025,15 +1208,22 @@ list(eph_iob *iob,int argc,char *argv[])
 		printf("%c ",res?'P':'-');
 		res=(long)(buffer[20]&0xff)+((long)(buffer[21]&0xff)<<8)+
 			((long)(buffer[22]&0xff)<<16)+((long)(buffer[23]&0xff)<<24);
-		if (res != -1L) printf("%8s\n",ctimetz(&res,usetimezone));
-		else printf("N/A\n");
+		if (res != -1L) printf("%8s",ctimetz(&res,usetimezone));
+		else printf("N/A");
+		if ((rc=eph_getvar(iob,79,&buffer,&bufsize)) == DC1) {
+			printf(" N/A\n");
+		} else if (rc) {
+			printf(" FAIL\n");
+		} else {
+			printf(" %s\n",buffer);
+		}
 	}
 	free(buffer);
 	return 0;
 }
 
 int
-llist(eph_iob *iob,int argc,char *argv[])
+llistfiles(eph_iob *iob, char *path)
 {
 	char *buffer,*p;
 	unsigned char *ress;
@@ -1045,8 +1235,13 @@ llist(eph_iob *iob,int argc,char *argv[])
 	if (eph_getint(iob,10,&max)) {
 		return -1;
 	}
-	printf("total %lu\n",max);
-	printf("No. P    Size R F Shuttr Date and Time\n");
+	if (have_folders) printf("%s: ",path);
+	if (max)
+		printf("total %lu\n"
+		"No. P    Size R F Shuttr Date and Time\n",
+			max);
+	else
+		printf("\n");
 
 	for (j=1;j<=max;j++) {
 		printf("%3ld ",j);
@@ -1124,6 +1319,24 @@ llist(eph_iob *iob,int argc,char *argv[])
 	return 0;
 }
 
+int
+list(eph_iob *iob,int argc,char *argv[])
+{
+	return descend(iob,0,"\\",listfiles);
+}
+
+int
+llist(eph_iob *iob,int argc,char *argv[])
+{
+	return descend(iob,0,"\\",llistfiles);
+}
+
+int
+cd(eph_iob *iob,int argc,char *argv[])
+{
+	return xchdir(iob,argv[1]);
+}
+
 static void
 reporterror(int errcode,char *errstr)
 {
@@ -1189,7 +1402,7 @@ retrfile(eph_iob *iob,int argc,char *argv[],int datatype)
 				((long)(buf8i[6]&0xff)<<16)+((long)(buf8i[7]&0xff)<<24);
 		} else
 			alength=0;
-		if (debug) fprintf(stderr,"Compressed audio length: %lu\n",
+		if (debug) printf("Compressed audio length: %lu\n",
 					alength);
 		filesize=length=alength;
 	} else {
@@ -1207,6 +1420,14 @@ retrfile(eph_iob *iob,int argc,char *argv[],int datatype)
 
 		ilength2=(long)(buf8i[0]&0xff)+((long)(buf8i[1]&0xff)<<8)+
 				((long)(buf8i[2]&0xff)<<16);
+		if (ilength2 && (ilength == 0L)) {
+			if (!quiet) fprintf(stderr,
+				"uh-oh, size of frame in reg12 (%ld) "
+				"is zero, will use size field from "
+				"picture summary register (%ld)!\n",
+				ilength,ilength2);
+			ilength=ilength2;
+		}
 		if (ilength == ilength2) {
 			pictime=(long)(buf8i[20]&0xff)+
 				((long)(buf8i[21]&0xff)<<8)+
@@ -1216,12 +1437,12 @@ retrfile(eph_iob *iob,int argc,char *argv[],int datatype)
 			if (!quiet) fprintf(stderr,
 				"uh-oh, size of frame in reg12 (%ld) "
 				"does not match size field from "
-				"picture summary regiser (%ld), ignore "
+				"picture summary register (%ld), ignore "
 				"summary data!\n",
 				ilength,ilength2);
 		}
 	}
-	if (debug) fprintf(stderr,"got timestamp from reg47: %lu\n",
+	if (debug) printf("got timestamp from reg47: %lu\n",
 				pictime);
 	free(buf8i);
 
@@ -1319,7 +1540,7 @@ retrfile(eph_iob *iob,int argc,char *argv[],int datatype)
 			if (sscanf(p,"%lu",&pictime) != 1)
 				pictime=(time_t)-1;
 		}
-		if (debug) fprintf(stderr,"got timestamp from file: %lu\n",
+		if (debug) printf("got timestamp from file: %lu\n",
 					pictime);
 	}
 	if (!quiet) {
@@ -1559,16 +1780,70 @@ off(eph_iob *iob,int argc,char *argv[])
 
 /* -------------------------------------------------------------------- */
 
+typedef struct _range {
+	int max;
+	char *bits;
+} range_t;
+
+static int
+setbit(range_t *range,int index)
+{
+	register int pos,shift;
+
+	if (index > range->max) {
+		char *tmp;
+		int newsize;
+
+		newsize=(index/128+1)*16;
+		tmp=(char *)malloc(newsize);
+		if (tmp == NULL) return -1;
+		memset(tmp,0,newsize);
+		memcpy(tmp,range->bits,(range->max)/8);
+		if (range->max) free(range->bits);
+		range->bits=tmp;
+		range->max=newsize*8;
+	}
+	pos=index/8;
+	shift=index%8;
+	range->bits[pos] |= (1<<shift);
+	return 0;
+}
+
+static int
+bitset(range_t *range,int index)
+{
+	register int pos,shift;
+
+	if ((index < 1) || (index > range->max)) return 0;
+	pos=index/8;
+	shift=index%8;
+	return ((range->bits[pos]) & (1<<shift))?1:0;
+}
+
+static void
+freebits(range_t *range)
+{
+	if (range->max) free(range->bits);
+	range->max=0;
+}
+
 int
 iterate(int (*executor)(eph_iob *iob,int argc,char *argv[]),
 		eph_iob *iob,int argc,char *argv[],int intvl,int filearg)
 {
 	int rc=0;
 	long bot,top,i,step;
-	char buf[16],*p;
+	char buf[16],*p,*intvlarg;
 	struct stat st;
+	range_t range;
+	int reorder=0;
 
 	if (intvl == 0) return (*executor)(iob,argc,argv);
+	if (intvl < 0) {
+		reorder=1;
+		intvl=-intvl;
+		range.max=0;
+	}
 
 	if (filearg &&
 	    (strchr(argv[intvl],',') || strchr(argv[intvl],'-'))) {
@@ -1578,7 +1853,12 @@ iterate(int (*executor)(eph_iob *iob,int argc,char *argv[]),
 		}
 	}
 
-	for (p=strtok(argv[intvl],",");p;p=strtok(NULL,",")) {
+	intvlarg=argv[intvl];
+	argv[intvl]=buf;	/* it's a bad taste to modify
+				arg vector, but we won't use
+				this argument anymore, so who
+				cares... */
+	for (p=strtok(intvlarg,",");p;p=strtok(NULL,",")) {
 		if (strcasecmp(p,"all") == 0) {
 			bot=1;
 			top=-1;
@@ -1607,16 +1887,27 @@ iterate(int (*executor)(eph_iob *iob,int argc,char *argv[]),
 		if (top >= bot) step=1;
 		else step=-1;
 		top+=step;
-		argv[intvl]=buf;	/* it's a bad taste to modify
-					arg vector, but we won't use
-					this argument anymore, so who
-					cares... */
 		for (i=bot;i!=top;i+=step) {
-			sprintf(buf,"%ld",i);
-			if ((rc=(*executor)(iob,argc,argv))) return rc;
+			if (reorder) {
+				if (setbit(&range,i)) {
+					fprintf(stderr,"no memory for range\n");
+					return -1;
+				}
+			} else {
+				sprintf(buf,"%ld",i);
+				if ((rc=(*executor)(iob,argc,argv))) return rc;
+			}
 		}
 	}
 
+	if (reorder) {
+		for (i=range.max;i>0;i--) if (bitset(&range,i)) {
+			sprintf(buf,"%ld",i);
+			if ((rc=(*executor)(iob,argc,argv))) break;
+		}
+		freebits(&range);
+	}
+
 	return rc;
 } 
 
@@ -1641,7 +1932,8 @@ struct _cmdlist {
 	{"picsum",	1,1,0,	picsum,		"-<frame-interval>"	},
 	{"allregs",	1,0,0,	allregs,	"-<max-reg-no>"			},
 	{"",		0,0,0,	NULL,		"== Do some actions =="	},
-	{"erase",	1,1,0,	erase,		"<frame-interval>"		},
+	{"cd",		1,0,0,	cd,		"<path>"		},
+	{"erase",	1,-1,0,	erase,		"<frame-interval>"		},
 	{"protect",	2,1,0,	protect,	"`On' | `Off' <frame-interval>"	},
 	{"snapshot",	0,0,0,	snapshot,	""			},
 	{"eraseall",	0,0,0,	eraseall,	""			},
@@ -1700,8 +1992,9 @@ showhelp(char *name)
 	printf("\t\t\t otherwise make file names using strftime(3),\n");
 	printf("\t\t\t if fmt contains `%%%%NNNd' then place count there\n");
 #endif
-	printf("\t-s baud\t- set port speed to 9600,19200,38400,57600 or 115200\n");
+	printf("\t-s baud\t- set communication speed to 9600,19200,38400,57600 or 115200\n");
 	printf("\t\t\tdefault is %d\n",MAX_SPEED);
+	printf("\t-S baud\t- set \"pseudo\" speed (see manual)\n");
 	printf("\t-l dev\t- use device name instead of default %s\n\n",device);
 	printf("Commands:\n");
 	for (i=0;cmdlist[i].cmd;i++) {
@@ -1757,7 +2050,7 @@ main(int argc,char *argv[])
 {
 	int c,i,rc=0;
 	char *cmd;
-	long speed=0;
+	long speed=0L,ttspeed=0L,defttspeed=0L;
 	eph_iob *iob;
 	char *program = argv[0];
 #ifdef MS_PROGRAM_NAMES
@@ -1801,14 +2094,28 @@ main(int argc,char *argv[])
 	}
 #endif /* HAVE_SCHED_SETSCHEDULER */
 
-	while ((c=getopt(argc,argv,"l:s:f:vqhVzt")) != EOF)
+#ifdef DEVVERSION
+	printf("Dev ver: %s\n",rcsid);
+#endif
+
+	while ((c=getopt(argc,argv,"l:s:S:f:vqhVzt")) != EOF)
 	switch (c) {
 	case 'l':
 		device=optarg;
 		break;
 	case 's':
+		/* real speed, physical in the line, tell the camera
+		   about it. */
 		speed=atol(optarg);
 		break;
+	case 'S':
+		/* `pseudo' speed, tell this to the UNIX serial
+		   driver so that it will set real speed on the chip */
+		if (sscanf(optarg,"%ld,%ld",&defttspeed,&ttspeed) != 2) {
+			fprintf(stderr,"bad `-S' setting, ignoring\n");
+			ttspeed=defttspeed=0L;
+		}
+		break;
 	case 'f':
 		nameformat=optarg;
 		break;
@@ -1874,13 +2181,14 @@ main(int argc,char *argv[])
 		fprintf(stderr,"eph_new failed\n");
 		return 1;
 	}
-	if (eph_open(iob,device,speed)) {
+
+	if (eph_open(iob,device,speed,defttspeed,ttspeed)) {
 		fprintf(stderr,"eph_open failed\n");
 		return 1;
 	}
 
-	if ((rc=probe(iob))) {
-		fprintf(stderr,"probe failed\n");
+	if ((rc=init(iob))) {
+		fprintf(stderr,"init failed\n");
 		goto exit;
 	}
 
